/*****
 *
 * css-cat.c
 * Copyright 1999 Derek Fawcus.
 *
 * This file is part of libcss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****/

/*
 * Decode selected sector types from a CSS encoded DVD to stdout.  Use as a
 * filter on the input to mpeg2player or ac3dec.
 */

#include <stdio.h>
#include <stdlib.h>
#if defined(__linux__)
# include <getopt.h>
#endif /* __linux__ */
// FIXME #include <string.h> conflicts with #include <linux/fs.h> (below)
//#include <string.h>  // FIXME this conflicts with #include <linux/fs.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include "css.h"

struct {
	char *dev_name;
	int do_all;
	int do_video;
	int do_ac3;
	int do_mpg;
	int keep_pack;
	int keep_pes;
} param = {
	NULL,
	0,
	0,
	0,
	0,
	0,
	-1,
};

#define STCODE(p,a,b,c,d) ((p)[0] == a && (p)[1] == b && (p)[2] == c && (p)[3] == d)

u_long long offset = 0x0;
u_long long offset_end = 0x249a4;

/**
 *
 */

static void super_seek (int fd, u_long long offset)
{
	int i;


	lseek (fd, offset%0x7FFFFFFF, SEEK_SET);

	for (i=0; i<offset/0x7FFFFFFF; i++)
		lseek (fd, 0x7FFFFFFF, SEEK_CUR);
}


/**
 *
 */

static void un_css (int fdi, int fdo, u_char *key_title)
{
	u_char sector_buf[2048];
	u_char	*sp=NULL,
		*pes=NULL;
	int	written,
		wr,
		peslen=0,
		hdrlen=0;

	super_seek (fdi, offset*2048);

	while ((read (fdi, sector_buf, 2048) == 2048)) {
		if (!STCODE (sector_buf,0x00,0x00,0x01,0xba)) {
			fputs ("Not Pack start code\n", stderr);
			continue;
		}

		if (param.do_all)
			goto write_it;

		pes = sector_buf + 14 + (sector_buf[13] & 0x07);

/* System Header Pack Layer */
		if (STCODE (pes,0x00,0x00,0x01,0xbb)) {
			peslen = (pes[0x04] << 8) + pes[0x05];
			pes += peslen + 6;
		}

		if (pes[0x00] || pes[0x01] || pes[0x02] != 0x01 || pes[0x03] < 0xbc) 
			continue;
		
		peslen = (pes[0x04] << 8) + pes[0x05];
		hdrlen = pes[0x08] + 6 + 3;
		if ((pes[0x03] & 0xf0) == 0xe0) {
			if (param.do_video)
				goto write_it;
		} else if (param.do_mpg && pes[0x03] == (0xc0 | (param.do_mpg - 1))) { /* MPEG Audio */
			goto write_it;
		} else if (pes[0x03] == 0xbd) { /* AC3 Audio */
			if (param.do_ac3) {
				int audiotrack = param.do_ac3 - 1;
				if (pes[hdrlen] == (0x80|(audiotrack & 7))) {
					hdrlen += 4;
					goto write_it;
				}
			}
		}

		continue;

	write_it:
		CSSDescramble (sector_buf, key_title);

		written = 0;
		if (param.keep_pack)
			sp = sector_buf, peslen = 2048;
		else if (param.keep_pes)
			sp = pes, peslen = 2048 - (pes - sector_buf);
		else
			sp = pes + hdrlen, peslen -= hdrlen - 6;

		do {
			wr = write (fdo, sp, peslen - written);
			sp += wr;
			written += wr;
		} while (wr > 0 && written < peslen);
	}
}


/**
 *
 */

static int parse_args(int argc, char **argv)
{
	int c;
	opterr = 0;
	param.dev_name = NULL;

	while (1)
		switch((c = getopt(argc, argv, "at:Ppvm:01234567"))) {
		case 'a':
			param.do_all = 1;
			/* fall through */
		case 'P':
			param.keep_pack = 1;
			break;
		case 'p':
			param.keep_pes = 1;
			break;
		case 't':
//			title = optarg;
			break;
		case 'v':
			param.do_video = 1;
			++param.keep_pes;
			break;
		case 'm':
			if ((param.do_mpg = atoi(optarg)) < 1 || param.do_mpg > 32)
			++param.keep_pes;
			break;
		case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8':
			param.do_ac3 = c - '0';
			++param.keep_pes;
			break;
		case EOF:
			goto got_args;
		default:
			break;
		}

got_args:
	param.keep_pes = (param.keep_pes > 0) ? 1 : 0;

	return optind;
}


/**
 *
 */

#include <linux/fs.h>

int path_to_lba (char *p)
{
	int fd, lba = 0;

	if ((fd = open (p, O_RDONLY)) < 0) {
		perror("DVD vob file");
		return -1;
	}

	if (ioctl(fd, FIBMAP, &lba) < 0) {
		perror ("ioctl FIBMAP");
		close(fd);
		return -1;
	}

	close(fd);

	return lba;
}


/**
 *
 */

int main (int argc, char *argv[])
{
	u_char key_disc[2048];
	u_char key_title[5];

	int fd;
	int ret=-1;
	int argi;

	if ((argi = parse_args (argc, argv)) >= argc);

#if 0
	if ((fd_disc = open (param.dev_name, O_RDONLY | O_NONBLOCK)) == -1) {
		perror ("can't open DVD");
		goto clean_up;
	}
#endif
	if ((fd = open (argv[argi], O_RDONLY)) == -1) {
		perror ("can't open DVD");
		goto clean_up;
	}

#if 1
{
	int key_fd;

	if ((key_fd = open ("disc-key", O_RDONLY)) == -1) {
		perror ("can't open disc-key");
		exit (1);
	}
	if (read (key_fd, key_disc, 2048) != 2048) {
		perror ("can't read disc-key");
		close (key_fd);
		goto clean_up;
	}

	if ((key_fd = open ("title-key", O_RDONLY)) == -1) {
		perror ("can't open title-key");
		goto clean_up;
	}
	if (read (key_fd, key_title, 5) != 5) {
		perror ("can't read title-key");
		close (key_fd);
		goto clean_up;
	}
	if (CSSDecryptTitleKey (key_title, key_disc) < 0) {
		perror ("Decrypting title");
		close (key_fd);
//		goto clean_up;
	}
}
#else
	if ((lba = path_to_lba (argv[argi])) < 0)
		return -1;

/***** authenticate  *****/

	if (CSSAuthDisc (fd_disc, key_disc) < 0) {
		perror ("Authenticate disc");
		goto clean_up;
	}
	if (CSSAuthTitle (fd_disc, key_title, lba) < 0) {
		perror ("Authenticate title");
		goto clean_up;
	}

	if (CSSDecryptTitleKey (key_title, key_disc) < 0) {
		perror ("Decrypting title");
//		goto clean_up;
	}
#endif
/*****/

#if (TRYALLKEYS) /* We figure if they're doing key discovery they're
                    not actually watching a DVD */
	goto clean_up;
#endif

	un_css (fd, 1, key_title);
	ret = 0;

clean_up:
	close(fd);

	return ret;
}
