/*****
 *
 * This file is part of libcss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****/

/*
 * A noddy program for getting and printing some info from the
 * DVD-ROM drive.
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#if defined(__NetBSD__) || defined(__OpenBSD__)
# include <sys/dvdio.h>
#elif defined(__linux__)
# include <linux/cdrom.h>
#else
# error "Need the DVD ioctls"
#endif
#include <sys/ioctl.h>
#include <errno.h>

#include "css.h"

#define DVD	"/dev/cdrom"

/**
 * get physical disc information
 **/

int GetPhysical(int fd)
{
	dvd_struct d;
	struct dvd_layer *layer;
	int i;
	int layers = 4;

	d.physical.type = DVD_STRUCT_PHYSICAL;
	for (i=0; i<layers; i++) {
		d.physical.layer_num = i;
	
		if (ioctl (fd, DVD_READ_STRUCT, &d)<0) {
			fprintf (stderr, "Could not read Physical layer %d\n", i);
			return 0;
		}

		layer = &d.physical.layer[i];
		layers = layer->nlayers + 1;

		printf ("Layer %d[%d]\n", i, layers);
		printf (" Book Version:   %d\n", layer->book_version);
		printf (" Book Type:      %d\n", layer->book_type);
		printf (" Min Rate:       %d\n", layer->min_rate);
		printf (" Disc Size:      %d\n", layer->disc_size);
		printf (" Layer Type:     %d\n", layer->layer_type);
		printf (" Track Path:     %d\n", layer->track_path);
		printf (" Num Layers:     %d\n", layer->nlayers);
		printf (" Track Density:  %d\n", layer->track_density);
		printf (" Linear Density: %d\n", layer->linear_density);
		printf (" BCA:            %d\n", layer->bca);
		printf (" Start Sector    %#x\n", layer->start_sector);
		printf (" End Sector      %#x\n", layer->end_sector);
		printf (" End Sector L0   %#x\n", layer->end_sector_l0);
	}

	return 1;
}


/**
 * get copyright information
 **/

int GetCopyright (int fd)
{
	dvd_struct d;

	d.copyright.type = DVD_STRUCT_COPYRIGHT;
	d.copyright.layer_num = 0;
	
	if (ioctl(fd, DVD_READ_STRUCT, &d)<0) {
		fprintf (stderr, "Could not read Copyright Struct\n");
		return 0;
	}

	printf ("Copyright: CPST=%d, RMI=%#02x\n", d.copyright.cpst, d.copyright.rmi);

	return 1;
}


/**
 * get encryption status of disc
 **/

int GetEncryption (int fd)
{
	dvd_struct dvd;

	dvd.copyright.type = DVD_STRUCT_COPYRIGHT;
	dvd.copyright.layer_num = 0;

	if (ioctl (fd, DVD_READ_STRUCT, &dvd) < 0) {
		perror ("DVD_READ_STRUCT");
		return -1;
	}

	printf ("Disc is %s\n", dvd.copyright.cpst ? "encrypted." : "NOT encrypted at all - no need to use CSS!!!");

	return dvd.copyright.cpst;
}


/**
 *
 **/

int main (int argc, char *argv[])
{
	int fd;
	char *device = strdup (DVD);

	if (argc > 1)
		device = strdup (argv[1]);

	fd = open(device, O_RDONLY | O_NONBLOCK);
	free (device);

	if (fd < 0) {
		fprintf (stderr, "unable to open dvd drive (%s).\n", device);
		return -1;
	}

	GetEncryption (fd);

	if (CSSGetASF (fd) < 0)
		return -1;

	GetPhysical (fd);
	GetCopyright (fd);

	return 0;
}
