/*****
 *
 * This file is part of libcss.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****/

/*
 * tstdvd.c
 *
 * Example program showing usage of DVD CSS ioctls
 *
 * Copyright (C) 1999 Andrew T. Veliath <andrewtv@usa.net>
 * See http://www.rpi.edu/~veliaa/linux-dvd for more info.
 */

/* Hacked about by Derek Fawcus <derek@spider.com> such that
 * it can be used as a simple program to authenticate the
 * computer with the DVD-ROM drive.
 *
 * If supplied with one parameter it gets the disc key and
 * saves it to a file.  If supplied with a second parameter
 * (a LBA) then it gets the title key for the supplied LBA.
 *
 * When getting the disc key,  only the first 10 bytes of it
 * are printed.  The whole key is written to the file.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>

#include "css.h"

#ifndef FIBMAP
#define FIBMAP 1
#endif

int path_to_lba(char *p)
{
	int fd, lba = 0;

	if ((fd = open(p, O_RDONLY)) == -1) {
		perror("DVD vob file");
		return 0;
	}
	if (ioctl(fd, FIBMAP, &lba) != 0) {
		perror ("ioctl FIBMAP");
		close(fd);
		return 0;
	}

	close(fd);

	return lba;
}

int main(int argc, char **argv)
{
	char *device;
	int fd, title = 0, lba = 0;
	char key_disc[2048];
	char key_title[5];
	int fdd;

	if (argc < 2) {
		fprintf(stderr, "usage: tstdvd <device> [title_path]\n");
		exit (1);
	}
	device = argv[1];
	fd = open (device, O_RDONLY | O_NONBLOCK);
	if (fd < 0) {
		perror(device);
		exit(1);
	}
	if (argc == 3) {
		lba = path_to_lba(argv[2]);

printf ("LBA: %x\n", lba);

		title = 1;
	}

/***** Disc Key *****/

	CSSAuthDisc (fd, key_disc);

	fdd = open("./disc-key", O_WRONLY|O_TRUNC|O_CREAT, 0644);
	if (fdd < 0)
		perror ("Can't create \"disc-key\"");
	else {
		if (write(fdd, key_disc, 2048) != 2048)
			printf("Can't write \"disc-key\"\n");
		close(fdd);
	}

/***** Title Key *****/

	if (title) {
		int fdd;

		CSSAuthTitle (fd, key_title, lba);

		fdd = open("title-key", O_WRONLY|O_TRUNC|O_CREAT, 0644);

		if (fdd < 0)
			printf("Can't create \"title-key\"\n");
		else {
			if (write(fdd, key_title, 5) != 5)
				printf("Can't write \"title-key\"\n");
			close(fdd);
		}
	}

/*****/

	close(fd);

	return 0;
}
